/*
 zenegalircd: a lirc daemon for the BOSE Zenega Set-Top-Box
Copyright (C) 2006 D.Herrendoerfer
    modified 2008 O. Tscherwitschke

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include <stdio.h>
#include <unistd.h>
#include <linux/input.h>

#include "zenegalircd.h"

char messagebuf[128];

typedef struct
{
    char*   name;
    int     repeat;
} KeyDef;


KeyDef keys[] = {
    [KEY_SCROLLLOCK] =  {"MUTE",        1},
    [KEY_1] =           {"1",           1},
    [KEY_2] =           {"2",           1},
    [KEY_3] =           {"3",           1},
    [KEY_4] =           {"4",           1},
    [KEY_5] =           {"5",           1},
    [KEY_6] =           {"6",           1},
    [KEY_7] =           {"7",           1},
    [KEY_8] =           {"8",           1},
    [KEY_9] =           {"9",           1},
    [KEY_0] =           {"0",           1},

    [KEY_KP4] =         {"*",           1},
    [KEY_KP6] =         {"#",           1},

    [KEY_KP2] =         {"INFO",        1},
    [KEY_KP8] =         {"EPG",         1},
    [KEY_BACKSPACE] =   {"DEL",         1},

    [KEY_KP0] =         {"MENU",        1},
    [KEY_F1] =          {"TONLINE",     1},
    [KEY_KP5] =         {"EXIT",        1},

    [KEY_UP] =          {"UP",          1},
    [KEY_DOWN] =        {"DOWN",        1},
    [KEY_LEFT] =        {"LEFT",        1},
    [KEY_RIGHT] =       {"RIGHT",       1},
    [KEY_KP7] =         {"CHUP",        1},
    [KEY_KP1] =         {"CHDOWN",      1},
    [KEY_KP9] =         {"VOLUP",       1},
    [KEY_KP3] =         {"VOLDOWN",     1},
    [KEY_ENTER] =       {"OK",          1},

    [KEY_F3] =          {"STOP",        0},
    [KEY_F4] =          {"PLAY",        0},
    [KEY_KPMINUS] =     {"HELP",        1},

    [KEY_F5] =          {"PREV",        1},
    [KEY_F6] =          {"REW",         1},
    [KEY_F7] =          {"FWD",         1},
    [KEY_F8] =          {"NEXT",        1},

    [KEY_F9] =          {"RED",         1},
    [KEY_F10] =         {"GREEN",       1},
    [KEY_F11] =         {"YELLOW",      1},
    [KEY_F12] =         {"BLUE",        1}
};



char *getKey(int efd)
{
    struct input_event ev;
    int rd;
    static int repeat = 0;
    static int prestop = 0;


    /* read one event at a time */
    if ((rd = read (efd, &ev, sizeof (struct input_event))) < sizeof (struct input_event))
        return NULL;

    /* handle only EV_KEY events, ignore the rest */
    if (ev.type == EV_KEY)
    {
        if (ev.value == 0)          /* key released, ignored */
            return NULL;
        else if (ev.value == 1)     /* key pressed */
            repeat = 0;
        else if (ev.value == 2)     /* key autorepeated */
            repeat++;


        if (ev.code >= (sizeof keys / sizeof keys[0])       /* index too large */
            || keys[ev.code].name == NULL)                  /* or key undefined */
        {
            sprintf(messagebuf, "000000000000%04x %02i KEY_%04x ZENEGA\n", ev.code, repeat, ev.code);
        }
        else        /* index valid and key defined */
        {
            char* name = keys[ev.code].name;

            if (ev.value == 2 && keys[ev.code].repeat == 0)  /* exit, if key should not be repeated */
                return NULL;

            if (ev.code == KEY_F3)          /* STOP */
            {
                if (prestop == 0)           /* PAUSE before STOP */
                {
                    name = "PAUSE";
                    prestop = 1;
                }
                else
                    prestop = 0;
            }
            else if (ev.code == KEY_F4)     /* PLAY */
            {
                if (prestop == 1)           /* If paused, unpause again */
                    name = "PAUSE";

                prestop = 0;
            }
            else if (ev.code == KEY_F5 || ev.code == KEY_F8)
                prestop = 0;

            sprintf(messagebuf, "000000000000%04x %02i %s ZENEGA\n", ev.code, repeat, name);
        }

        return messagebuf;
    }

    return NULL;
}
